from . import GlyphItem


class LayoutRun(GlyphItem):
    """
    The :class:`LayoutRun` structure represents a single run within a
    :class:`LayoutLine`; it is simply an alternate name for :class:`GlyphItem`.
    See the :class:`GlyphItem` docs for details on the fields.
    """
    pass
